C*****************************************
       program gbcd from 3D 
C*****************************************
       implicit none
	   include 'common.fi'

       !parameter (CD=9)
       !parameter (CD2=9)
       !dimension gbd(CD,CD,CD,CD2,4*CD2)
       dimension gbd(12,12,12,12,48)
       dimension e1(3), e2(3)
       dimension gg1(3,3), gg2(3,3), dg(3,3)
       dimension so1(3,3), so2(3,3), sa(2), so2t(3,3)

       write(6,*) '=============================================='
	   write(6,*) ' PROGRAM TO CALCULATE THE FIVE PARAMETER'
	   write(6,*) ' GRAIN BOUNDARY CHARACTER DISTRIBUTION '
	   write(6,*) ' FROM THREE DIMENSIONAL EBSD DATA'
	   write(6,*)
	   write(6,*) ' DEVELOPED BY THE CMU MRSEC'
	   write(6,*) '  version 06/01/2010'
	   write(6,*) '  rohrer@cmu.edu '
       write(6,*) '=============================================='
	   
	   version = 'version 06/01/10'	   
	   
       pi = 4.0*atan(1.0)
	   eps = 1.e-6
	   HCD = 90.0

c  The section reads information from the input file, including the data file name,
c  the symmetry type.

 5     format(A)
       open(21, file='input.txt', status='old')
	   read(21,*) tempword
	   read(21,*) keyword
	   read(21,*) tempword, tempword, tempword
       read(21,*) msym, rotation, radian
	   read(21,*) tempword   
	   read(21,*) CD, CD2	 	   	   
	   read(21,5) inline2
	   	   
 10     format(A,A)
	    write(6,10)'I am going to look for data in the file labeled: ', keyword
 11     format(A,I2,2X,A,I2,2X,A,I2)
	    write(6,11)'msym =',msym,'rotation =',rotation,'radian =',radian
	   
	    close(21)

c  The statement calls the subroutine that contains the symmetry operators.  The variable
c  msym, read from the input file, specifies the symmetry (1 = tetragonal, 2 = hexagonal, 
c  3 = subic, 4 = trigonal), O is a matrix containing the operators, and nsymm is the 
c  number of symmetry operators.  The routine returns O and nsymm.

       call get_symop (msym, O, nsymm)

c  This part is to zero the grain boundary character distribution matrix		
		do i=1,CD
	     do j=1,CD
	      do k=1,CD
	       do ii=1,CD2
	        do jj=1,4*CD2
	         gbd(i,j,k,ii,jj)=0.0
			enddo
	       enddo
		  enddo
	     enddo
	    enddo	
		

c  This part is to determines the total number of line segments (lines in the data file),
c  assigned to the variable, nnline
		
		nnline = 0
		open(41, file=keyword, status='old')
 100    continue
		read(41,*,end=101)
		nnline=nnline+1
		goto 100
 101    continue
        close(41)
102		format(A,I7,A)
		write(6,102) 'There are ',nnline,' lines in the data file.'


c Check each line for the string '#'. If found, we know this line is part  
c of the header.  If not, we have reached the data section.		
	   open(41, file=keyword, status='old')
       header = 0
       hash = 0
	   do i=1,nnline
	    read(41, "(a)") inline
	    hash = index(inline, '#')
	    if (hash.ne.0) then
	     header=header+1
		 hash=0
	    else
		 goto 105     
	    endif			
	   enddo
 105   close(41)		

		
		open(41, file=keyword, status='old')
		open (32, file='gbcd.txt', status='unknown')
		nline = 0
		
		
		 do i=1,header
	      read(41, "(a)") inline
		  write(32, "(a)") inline
	     enddo	
		 write(32, "(a)") inline2
		
 110    continue
        nline=nline+1		

c  This section reads a single line from the data file
		
		read(41,*,end=200) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	nl(1), nl(2), nl(3), area

c        nl(3)=-nl(3)

c   This converts from the TSL reference frame
c        temp=-nl(1)
c	    nl(1)=-nl(2)   
c		nl(2)=temp
c  If the data are in the HKL reference frame, and the system is hexagonal,
c  then the data are rotated by -30 so that [2-1-10] is parallel to +x in 
c  the output (this direction is to the right in the plane of the paper)
		
	    if (rotation.eq.0.and.msym.eq.2) then
	     if (e1(3).gt.(30*pi/180)) then
 	      e1(3) = e1(3)-(30*pi/180)
		 else
		  e1(3) = e1(3)+(330*pi/180)
		 endif
		 if (e2(3).gt.(30*pi/180)) then
 	      e2(3) = e2(3)-(30*pi/180)
		 else
		  e2(3) = e2(3)+(330*pi/180)
		 endif
		endif
		
c  This part converts angular data to radians, if specified in the input file
c  by the variable, radian
				
	    if (radian.eq.0) then
	     e1(1)=e1(1)*(pi/180.0)
	     e1(2)=e1(2)*(pi/180.0)
	     e1(3)=e1(3)*(pi/180.0)

	     e2(1)=e2(1)*(pi/180.0)
	     e2(2)=e2(2)*(pi/180.0)
	     e2(3)=e2(3)*(pi/180.0)
	    endif

c  This part converts the Euler angles from the input file to a g matrix

        call EToG(e1,g1)
		call EToG(e2,g2)		
		
        do i=1,nsymm
c         this call returns the i-th 3x3 symmetry operator from O		
		 call symop(O,i,so1)
c         this hits the first g matrix with the i-th symmetry operator		  
		 call MToM (so1,g1,gg1)

          do j=1,nsymm
c		  this call returns the j-th 3x3 symmetry operator from O
		   call symop(O,j,so2)
c		  this hits the second g matrix with the j-th symmetry operator		   
		   call MToM(so2,g2,gg2)
c		  this takes the transpose of the second g matrix	
		   call trans(gg2,gt)
c		  this computes the misorientation between g1 and g2		   
		   call MToM(gg1,gt,dg)
c		  this returns three Euler angles for the misorientation across this segment
		   call GToE(dg,mort)
		  
c         this condition considers only misorientations when all 3 angles are less than 90.
c         if so, the three misorientation indices in the GBCD matrix are determined (c1,c2,c3)		  
		   if (mort(1).lt.pi*0.5.and.mort(2).lt.pi*0.5.and.mort(3).lt.pi*0.5) then
     		c1=int(float(CD)*2.0*mort(1)/pi)+1
			c2=int(float(CD)*cos(mort(2)))+1
			c3=int(float(CD)*2.0*mort(3)/pi)+1
c			replaced 1/4/07: corrected errors in the binning procedure
c                            c1=int(floor(float(CD)*2.0*mort(1))/pi)+1
c                            c2=int(floor(float(CD)*cos(mort(2))))+1
c                            c3=int(floor(float(CD)*2.0*mort(3))/pi)+1
			 if (c1.eq.CD+1) c1=CD
			 if (c2.eq.CD+1) c2=CD
			 if (c3.eq.CD+1) c3=CD

c          the section puts the line segments into bins cooresponding to orientations normal to the trace
c          it does this in one degree increments, from 1 to 90.


			  call MToV (gg1, nl, nf)
			  call VToAngles (nf, sa)
	          c4=int(float(CD2)*cos(sa(1)))+1
			  c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1		  
c			replaced 1/4/07: to simplify the binning procedure
c			      c4=int(floor(float(CD2)*cos(sa(1))))+1
c			      c5=int(floor(4.0*float(CD2)*sa(2)/(2.0*pi)))+1
			  if (c4.eq.CD2+1) c4=CD2
			  if (c4.eq.0) c4=1
			  if (c5.eq.4*CD2+1) c5=4*CD2
			  gbd(c1,c2,c3,c4,c5)=gbd(c1,c2,c3,c4,c5)+area

			endif

c		  this takes the transpose of the first g matrix			
		   call trans (gg1, gt)
c		  this computes the misorientation between g2 and g1		   
		   call MToM (gg2, gt, dg)
c		  this returns three Euler angles for the misorientation across this segment		   
		   call GToE (dg, mort)
c         this repeats the operations above for the interchanged bicrystal		
			if (mort(1).lt.pi*0.5 .and.mort(2).lt.pi*0.5.and.mort(3).lt.pi*0.5) then
     		c1=int(float(CD)*2.0*mort(1)/pi)+1
			c2=int(float(CD)*cos(mort(2)))+1
			c3=int(float(CD)*2.0*mort(3)/pi)+1
c			replaced 1/4/07: corrected errors in the binning procedure
c                            c1=int(floor(float(CD)*2.0*mort(1))/pi)+1
c                            c2=int(floor(float(CD)*cos(mort(2))))+1
c                            c3=int(floor(float(CD)*2.0*mort(3))/pi)+1
			if (c1.eq.CD+1) c1=CD
			if (c2.eq.CD+1) c2=CD
			if (c3.eq.CD+1) c3=CD



			call MToV (gg2, nl, nf)
			call VToAngles (nf, sa)
			c4=int(float(CD2)*cos(sa(1)))+1
			c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1		  
c			replaced 1/4/07: to simplify the binning procedure
c			      c4=int(floor(float(CD2)*cos(sa(1))))+1
c			      c5=int(floor(4.0*float(CD2)*sa(2)/(2.0*pi)))+1
			if (c4.eq.CD2+1) c4=CD2
			if (c4.eq.0) c4=1
			if (c5.eq.4*CD2+1) c5=4*CD2
			gbd(c1,c2,c3,c4,c5)=gbd(c1,c2,c3,c4,c5)+area

			endif

          enddo
        enddo

c  this just notifies you of the progress of the calculation
		if (mod(nline,5000).eq.0) then
	    write(6,*) nline,' of ',nnline
c		write(6,*) gbd(2,2,2,2,2)
	    endif

	    goto 110
		
c  the program comes to line 200 when it reaches the last line of the data file.		
 200    continue		
		

c  this loop determines the average area in each bin		
	    avg=0.0
	    do i=1,CD
	     do j=1,CD
	      do k=1,CD
	       do ii=1,CD2
	        do jj=1,4*CD2
	         avg=avg+gbd(i,j,k,ii,jj)
	        enddo
		   enddo
	      enddo
	     enddo
	    enddo
        
		avg=avg/float(4*CD*CD*CD*CD2*CD2)

 411    format(A,A)		
       write(32,411)'# gbcd file written by calc_gbcd: ',version
       write(32,411)'# based on file: ',keyword		
	   write(32,411)'# comment: ',inline2			

c  write out the MRD values in the file gbcd.txt

	    do i=1,CD
	     do j=1,CD
	      do k=1,CD
	       do ii=1,CD2
	        do jj=1,4*CD2
	         gbd(i,j,k,ii,jj)=gbd(i,j,k,ii,jj)/avg
	         write (32,*) gbd(i,j,k,ii,jj)
	        enddo
	       enddo
	      enddo
	     enddo
	    enddo
	    close (32)

       end